from machine import Pin, I2C
from time import sleep

sda=machine.Pin(20)
scl=machine.Pin(21)
i2c=machine.I2C(0, sda=sda, scl=scl, freq=400000)

font = {
    48: [0x3e, 0x51, 0x49, 0x45, 0x3e],  # 0
    49: [0x00, 0x42, 0x7f, 0x40, 0x00],  # 1
    50: [0x42, 0x61, 0x51, 0x49, 0x46],  # 2
    51: [0x21, 0x41, 0x45, 0x4b, 0x31],  # 3
    52: [0x18, 0x14, 0x12, 0x7f, 0x10],  # 4
    53: [0x27, 0x45, 0x45, 0x45, 0x39],  # 5
    54: [0x3c, 0x4a, 0x49, 0x49, 0x30],  # 6
    55: [0x01, 0x71, 0x09, 0x05, 0x03],  # 7
    56: [0x36, 0x49, 0x49, 0x49, 0x36],  # 8
    57: [0x06, 0x49, 0x49, 0x29, 0x1e],  # 9

}

MODE = 0b00011000
OPTS = 0b00001110  # 1110 = 35mA, 0000 = 40mA

CMD_BRIGHTNESS = 0x19
CMD_MODE = 0x00
CMD_UPDATE = 0x0C
CMD_OPTIONS = 0x0D

CMD_MATRIX_L = 0x0E
CMD_MATRIX_R = 0x01

brightness = 100
address = 0x61

# Caractère gauche
buf_matrix_left = bytearray ( [
# Row  7654321
    0b01111111,  # col 1, bottom = msb
    0b01111111,  # col 2
    0b01111111,  # col 3
    0b01111111,  # col 4
    0b01111111,  # col 5
    0b00000000,
    0b00000000,
    0b01000000   # bit 7 = decimal dot
])

# Caractère droit
buf_matrix_right = bytearray ( [
# Col    12345
    0b00011111,  # row 1
    0b00011111,  # row 2
    0b00011111,  # row 3
    0b00011111,  # row 4
    0b00011111,  # row 5
    0b00011111,  # row 6
    0b10011111,  # row 7 + bit 8 = decimal dot
    0b00000000
])

def clear():
# il faut appeler show() pour appliquer les modifs
    global buf_matrix_left
    global buf_matrix_right
    buf_matrix_left = bytearray ([ 0,0,0,0,0,0,0,0])
    buf_matrix_right = bytearray ([ 0,0,0,0,0,0,0,0])

def set_brightness(brightness, update=False):
# Définit la luminisité des LED.

#        :param brightnes: LED brightness from 0.0 to 1.0
#        :param update: Push change to display immediately (otherwise you must call .show())
    brightness = int(brightness * 127.0)
    brightness = min(127, max(0, brightness))
    if update:
        i2c.writeto_mem(address, CMD_BRIGHTNESS, bytearray([brightness & 0xFF]))

def set_decimal(left=None, right=None):
# Affiche les points décimaux

#        :param left: State of left decimal dot
#        :param right: State of right decimal dot
    if left is not None:
        if left:
            buf_matrix_left[7] |= 0b01000000
        else:
            buf_matrix_left[7] &= 0b10111111
    if right is not None:
        if right:
            buf_matrix_right[6] |= 0b10000000
        else:
            buf_matrix_right[6] &= 0b01111111

def set_pixel(x, y, c):
# Allume un seul pixel

#        :param x: x position from 0 to 9 (0-4 on left matrix, 5-9 on right)
#        :param y: y position
#        :param c: state on/off
    if x < 5:  # Left Matrix
        if c:
            buf_matrix_left[x] |= (0b1 << y)
        else:
            buf_matrix_left[x] &= ~(0b1 << y)
    else:      # Right Matrix
        x -= 5
        if c:
            buf_matrix_right[y] |= (0b1 << x)
        else:
            buf_matrix_right[y] &= ~(0b1 << x)

def set_character(x, char):
# Affcihe un caractère

#        :param x: x position, 0 for left, 5 for right, or in between if you fancy
#        :param char: string character or char ordinal
    if type(char) is not int:
        char = ord(char)
    char = font[char]
    for cx in range(5):
        for cy in range(8):
            c = char[cx] & (0b1 << cy)
            set_pixel(x + cx, cy, c)

def get_shape():
# Définit la taille de la matrice
    return 10, 7

def set_image(image, offset_x=0, offset_y=0, wrap=False, bg=0):
# Met une image PIL dans le buffer
    image_width, image_height = image.size

    if image.mode != "1":
        image = image.convert("1")
  
    display_width, display_height = get_shape()

    for y in range(display_height):
        for x in range(display_width):
            p = bg
            i_x = x + offset_x
            i_y = y + offset_y
            if wrap:
                while i_x >= image_width:
                    i_x -= image_width
                while i_y >= image_height:
                    i_y -= image_height
            if i_x < image_width and i_y < image_height:
                p = image.getpixel((i_x, i_y))
            set_pixel(x, y, p)

def show():
# Affiche les buffers sur les matrices de LED
    i2c.writeto_mem(address, CMD_MATRIX_L, buf_matrix_left)
    i2c.writeto_mem(address, CMD_MATRIX_R, buf_matrix_right)
    i2c.writeto_mem(address, CMD_MODE, bytearray([MODE & 0xFF]))
    i2c.writeto_mem(address, CMD_OPTIONS, bytearray([OPTS & 0xFF]))
    i2c.writeto_mem(address, CMD_BRIGHTNESS, bytearray([brightness & 0xFF]))
    i2c.writeto_mem(address, CMD_UPDATE, bytearray([0x01 & 0xFF]))


# Tester si la matrice est présente
print('Scan du bus i2c...')
devices = i2c.scan()

if len(devices) == 0:
    print("Aucun composant détecté !")
else:
    print('Composant(s) détectés :',len(devices))

for device in devices:
    print("Adresse en décimal : ",device," | Adresse en Hexadécimal : ",hex(device))

# Initialiser variables
compteur = 0   # Compteur
point =1       # Etat du point décimal

# Eteindre les points décimaux
set_decimal(0,0)

# Boucle infinie
while True:
    # Transformer le compteur en chaine de caractères
    if compteur < 10:
        right = str(compteur)
        left = "0"
    else :
        display = str(compteur)
        left, right = display
    
    # Mettre les caractères correspondants dans les buffers
    set_character(0, left)
    set_character(5, right)
    # Afficher les buffers
    show()
    #Incrémenter le compteur
    compteur += 1
    # Faire clignoter points décimaux
    if point ==1:
        point=0
    else:
        point = 1
    if point == 1:
        set_decimal(0,1)
    else :
        set_decimal(1, 0)
    # Quand le compteur atteint 99
    if (compteur > 99) :
        compteur = 0
        # Eteindre les LED
        clear()
        show()
        # Attendre 2 secondes
        sleep (2)
    sleep(0.25)
